<?php
/********************************* PHP-P10 ******************************
 *    P10 uplink class by pk910   (c)2011 pk910                         *
 ************************************************************************
 *                          Version 2 (OOP)                             *
 *                                                                      *
 * PHP-P10 is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with PHP-P10; if not, write to the Free Software Foundation,   *
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.       *
 *                                                                      *
 ************************************************************************
 * 
 *  ModCMD/ModCMD.class.php
 *
 * shares the incoming events to all bot's that request them.
 *
 ************************************************************************
 * 
 * 
 */
require_once("Binding.class.php");
 
$bindid = 1;
define("BIND_NEWSERVER", $bindid++);
define("BIND_SQUIT", $bindid++);
define("BIND_CONNECT", $bindid++);
define("BIND_NICK", $bindid++);
define("BIND_USERMODE", $bindid++);
define("BIND_QUIT", $bindid++);
define("BIND_JOIN", $bindid++);
define("BIND_PART", $bindid++);
define("BIND_KICK", $bindid++);
define("BIND_CHANMODE", $bindid++);
define("BIND_CHANMSG", $bindid++);
define("BIND_PRIVMSG", $bindid++);
define("BIND_CHANNOTICE", $bindid++);
define("BIND_PRIVNOTICE", $bindid++);
define("BIND_PREPARSE", $bindid++);
define("BIND_UNKNOWNCMD", $bindid++);

 
class ModCMD implements EventHandler {
	private static $eventHandler = null;
	private static $bindings = array();
	
	public static function getEventHandler() {
		if(self::$eventHandler == null) {
			self::$eventHandler = new ModCMD();
		}
		return self::$eventHandler;
	}
	
	public static function bind($bot, $type, $method) {
		if(is_a($bot, "Bot") && method_exists($bot, $method)) {
			if(array_key_exists($type, self::$bindings)) {
				foreach(self::$bindings[$type] as $binding) {
					if($binding->match($bot, $method))
						return;
				}
			} else 
				self::$bindings[$type] = array();
			self::$bindings[$type][] = new Binding($bot, $method);
		}
	}
	
	public static function unbind($bot, $type, $method) {
		if(is_a($bot, "Bot")) {
			if(array_key_exists($type, self::$bindings)) {
				foreach(self::$bindings[$type] as $id => $binding) {
					if($binding->match($bot, $method)) {
						unset(self::$bindings[$type][$id]);
						break;
					}
				}
			}
		}
	}
	
	public static function unbindBot($bot) {
		if(is_a($bot, "Bot")) {
			foreach(self::$bindings as $type => $bindings) {
				foreach($bindings as $id => $binding) {
					if($binding->match($bot, null)) {
						unset(self::$bindings[$type][$id]);
					}
				}
			}
		}
	}
	
	/********************************************************************************************
	 *                                       EVENT HANDLER                                      *
	 ********************************************************************************************/
	//All events trigger a binding. You may set the same binding for eg. CHANMODE and USERMODE (or some others)
	//to get only one binding for both events... but i think that's senceless
	const EVENT_NEWSERVER  = BIND_NEWSERVER;
	const EVENT_SQUIT      = BIND_SQUIT;
	const EVENT_CONNECT    = BIND_CONNECT;
	const EVENT_NICK       = BIND_NICK;
	const EVENT_USERMODE   = BIND_USERMODE;
	const EVENT_QUIT       = BIND_QUIT;
	const EVENT_JOIN       = BIND_JOIN;
	const EVENT_PART       = BIND_PART;
	const EVENT_KICK       = BIND_KICK;
	const EVENT_CHANMODE   = BIND_CHANMODE;
	const EVENT_CHANMSG    = BIND_CHANMSG;
	const EVENT_PRIVMSG    = BIND_PRIVMSG;
	const EVENT_CHANNOTICE = BIND_CHANNOTICE;
	const EVENT_PRIVNOTICE = BIND_PRIVNOTICE;
	const EVENT_PREPARSE   = BIND_PREPARSE;
	const EVENT_UNKNOWNCMD = BIND_UNKNOWNCMD;
	
	private function event($type, $parameters) {
		if(array_key_exists($type, self::$bindings)) {
			foreach(self::$bindings as $binding) {
				$binding->trigger($parameters);
			}
		}
	}
	
	public function event_newserver($server, $isBurst) {
		$this->event(self::EVENT_NEWSERVER, array($server, $isBurst));
	}
	
	public function event_squit($server){
		$this->event(self::EVENT_SQUIT, array($server));
	}
	
	public function event_connect($user, $isBurst) {
		$this->event(self::EVENT_CONNECT, array($user, $isBurst));
	}
	
	public function event_nick($user, $newNick) {
		$this->event(self::EVENT_NICK, array($user, $newNick));
	}
	
	public function event_usermode($user, $modes) {
		$this->event(self::EVENT_USERMODE, array($user, $modes));
	}
	
	public function event_quit($user, $reason) {
		$this->event(self::EVENT_QUIT, array($user, $reason));
	}
	
	public function event_join($user, $channel, $isBurst) {
		$this->event(self::EVENT_JOIN, array($user, $channel, $isBurst));
	}
	
	public function event_part($user, $channel, $reason) {
		$this->event(self::EVENT_PART, array($user, $channel, $reason));
	}
	
	public function event_kick($user, $target, $channel, $reason) {
		$this->event(self::EVENT_KICK, array($user, $target, $channel, $reason));
	}
	
	public function event_chanmode($user, $channel, $modes) {
		$this->event(self::EVENT_CHANMODE, array($user, $channel, $modes));
	}
	
	public function event_chanmessage($user, $channel, $message) {
		$this->event(self::EVENT_CHANMSG, array($user, $channel, $message));
	}
	
	public function event_channotice($user, $channel, $message) {
		$this->event(self::EVENT_CHANNOTICE, array($user, $channel, $message));
	}
	
	public function event_privmessage($user, $channel, $message) {
		$this->event(self::EVENT_PRIVMSG, array($user, $channel, $message));
	}
	
	public function event_privnotice($user, $channel, $message) {
		$this->event(self::EVENT_PRIVNOTICE, array($user, $channel, $message));
	}
	
	public function event_preparse($from, $command, $arguments) {
		$this->event(self::EVENT_PREPARSE, array($from, $command, $arguments));
	}
	
	public function event_unknown_cmd($from, $command, $arguments) {
		$this->event(self::EVENT_UNKNOWNCMD, array($from, $command, $arguments));
	}
	
}

?>